;Filename: Feelr.s
;Part: ADuC702x.
;Created: 26/05/2004
;Updated: 28/06/2005
;Author: Eckart Hartmann.
;This file is in beta sampling.

#include        <ADuC702x.IAR.equ>

; FeeClr==========Erases pages of flash.
; C Function prototype: int FeeClr(int iAddr, int iPages);
;iAddr:{0-0xf7ff}
;iPages:{0-124}
; Description of Function: Starting with the flash page 
;       that contains iAddr erase iPages pages.
; User interface:
;   Set iAddr to an address within the first page to erase.
;   Set iPages to the number of sequential pages to erase.
;   Call FeeClr(iAddr, iPages).
;   If iAddr > 0xfdff erase nothing.
;   Else if iAddr<512 (e.g. 0) and iPages==0 the entire 
;    flash is erased. 
;   Else starting with the flash page that contains iAddr 
;    erase iPages pages except pages that are protected. 
;   Returns number of pages actually erased.
; Robustness:
;  Page erasure takes about 20ms per page in which time 
;    any access to flash will stall the core.
;   Obviously erasing a wrong page can be disastrous.
; Side effects: Corrupts r1, r2 and r3.

                RSEG CODE
                MODULE FeeClr
                PUBLIC FeeClr
                CODE32

FeeClr:
        stmfd   sp!,{r4,lr}
        cmp     r0,#0xf800              ;r0 = iAddr&0xf800;
        mov     lr,#0                   ;if(r0>0xf800)
        bhi     RetFC                   ; return iRet = 0;
        ands    r0,r0,#0xfe00           ;iAddr &= 0xfe00;
        cmpeq   r1,#0                   ;if(!iAddr && !iPages)
        moveq   r1,#124                 ; iPages = 124;
        add     r1,r0,r1,LSL#9          ;iEnd = iAddr+(iPages<<9);
NxtFC:  cmp     r0,r1                   ;while(r0<r1)
        bhs     RetFC                   ; {
        cmp     r0,#0xf800              ; if(r0<0x8f800)
        bcs     SkpFC                   ;  {
        mov     r2,r0,LSR#11            ;  r2 = 1<<(r0/0x800);
        mov     r3,#1
        mov     r2,r3,LSL r2
        mov     r3,#0xff000000          ;r3 = F_BASE;
        mov     r3,r3,ASR#12
        ldr     r4,[r3,#F_FEEPRO]       ;  if(!(r2&FEEPRO))
        tst     r4,r2                   ;   {
        bne     SkpFC
        str     r0,[r3,#F_FEEADR]       ;   FEEADR = r0;
        mov     r2,#5                   ;   FEECON = 5;
        str     r2,[r3,#F_FEECON]
        add     lr,lr,#1                ;   iPageCnt++;
ChkEDn: ldr     r2,[r3,#F_FEESTA]       ;   while(!FEESTA&1);
        tst     r2,#1
        beq     ChkEDn                  ;  }}
SkpFC:  add     r0,r0,#0x200            ; r0 += 0x200;
        b       NxtFC                   ; }
RetFC:  mov     r0,lr                   ;return iRet;
        ldmfd   sp!,{r4,lr}
        bx      lr                   ;return


                 ENDMOD FeeClr 

                 END
        
